/*----------------------------------------------------------------------------*/
/* app0to9.c - application dependent routines, sample problem using non-binary*/
/*             alphabet, testing program for solution by GALOPPS.  Fitness    */
/*             has optimum when value = fieldnumber % alpha_size (i.e., if    */
/*             alpha_size = 4, best soln is 0,1,2,3,0,1,2,3,0,1,... etc. This */
/*             is a simple, non-deceptive, non-epistatic "counting ones"-type */
/*             illustrative problem, so GA can be run "flat out" -- uniform   */
/*             crossover, high rates, radical or no scaling, etc., but any    */
/*             other optimization method can solve this simple problem even   */
/*             faster.     */
/*----------------------------------------------------------------------------*/


#include "stdafx.h"
#include "MSetar.h"
#include "MSetarDlg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "math.h"

#include "external.h"
#include "AllDinamica.h"



/* Application-dependent objective function.  THIS IS THE ONLY ROUTINE THE    */
/* USER ABSOLUTELY MUST FILL IN TO DEFINE THE USER'S PROBLEM.  OTHERS ARE     */
/* OPTIONAL, DEPENDING ON THE NATURE AND COMPLEXITY OF THE PROBLEM.           */

/* This is where you code the objective function for your particular problem, */
/* getting the genotype from critter->chrom, and then placing the             */
/* raw (unscaled) fitness for critter into critter->init_fitness.             */
/* Another action which must be accomplished is to increment the current      */
/* evaluation number (neval) and record it in critter->neval.                 */

void objfunc(struct individual *critter) {

	int i, j, which_serie, n_regime, which_serie_1, which_serie_2, pos_thres_1, pos_thres_2, ritardo, ritardo1, ritardo2;
	int codify[16], type_model;
	double value_fitness, sum=0.0;
	int *pos_threshold, yes_print=0;
	FILE *cfPtr;


	pos_threshold = ivector(0,4);
	neval++;
    local_cycle_neval++;

    critter->neval = neval;

	if(yes_print==1) {
		if((cfPtr = fopen ("objfunc.txt", "a")) ==NULL) printf("File could not be opened\n");
	}


    for (i=0;i<numfields;i++) {
	codify[i] = getfield(i, numfields, critter->chrom);
	if(yes_print==1) fprintf(cfPtr, "codify[%d]= %d\n",i,codify[i]); 
	}

	// Se vogliamo Esamininare  solo modelli con soglia bivariata
	//codify[0] = 1;
	//codify[2] = 0;
	//codify[3] = 1;
	//codify[10]= 0;


	if(codify[0] == 0) { // Modelli con una soglia univariata
		which_serie = codify[1]+1;
		if(yes_print==1) fprintf(cfPtr, "which_serie= %d\n",which_serie);
		n_regime    =  codify[4]+1;
		if(codify[4]==0) n_regime =2;
		for (j=0;j<n_regime-1;j++){
			pos_threshold[j] = codify[5+j];
			if(yes_print==1) fprintf(cfPtr, "pos_threshold[%d]= %d\n",j,pos_threshold[j]);
		}
		ritardo=codify[13]+1;
		value_fitness = calcule_fitness_1(which_serie,n_regime,pos_threshold,0, ritardo);
	}
	else { // Modelli con soglia bivariata
		
		which_serie_1 = codify[2]+1;
		which_serie_2 = codify[3]+1;
				
		//se le componenti della soglia bivariata sono le le stesse otteniamo un modello del tipo sopra
		if(which_serie_1 == which_serie_2) {
			/*which_serie =which_serie_1; 
			n_regime    =  codify[4]+1;
			if(yes_print==1) fprintf(cfPtr, "n_regime= %d\n",n_regime);
		    for (j=0;j<n_regime-1;j++){
				pos_threshold[j] = codify[5+j];
				if(yes_print==1) fprintf(cfPtr, "pos_threshold[%d]= %d\n",j,pos_threshold[j]);
			}
			ritardo=codify[13]+1;
			value_fitness = calcule_fitness_1(which_serie,n_regime,pos_threshold,0, ritardo);*/
			if(which_serie_1==1) which_serie_2=2;
			if(which_serie_1==2) {
				which_serie_1=1;
				
			}
			if(yes_print==1) fprintf(cfPtr, "which_serie1= %d\n",which_serie_1);
			if(yes_print==1) fprintf(cfPtr, "which_serie2= %d\n",which_serie_2);
			pos_thres_1 = codify[8];
			pos_thres_2 = codify[9];
			if(yes_print==1) fprintf(cfPtr, "pos_thres_1= %d\n",pos_thres_1);
			if(yes_print==1) fprintf(cfPtr, "pos_thres_2= %d\n",pos_thres_2);
			if(codify[10]==0) {
				if(yes_print==1) fprintf(cfPtr, "Modello con 4 regimi\n");
				type_model =0;
				n_regime =4;
				ritardo1=codify[14]+1;
                ritardo2=codify[15]+1;
				if(yes_print==1) fprintf(cfPtr, "Ritardo1= %d\n",ritardo1);
			    if(yes_print==1) fprintf(cfPtr, "Ritardo2= %d\n",ritardo2);
				value_fitness = calcule_fitness_2(which_serie_1,which_serie_2, pos_thres_1,pos_thres_2,n_regime, type_model,0, ritardo1, ritardo2);
			}
			if(codify[10]==1) {
				type_model =codify[11];
				if(yes_print==1) fprintf(cfPtr, "type_model=%d\n", type_model);
				if(type_model==0) n_regime =4;
				else n_regime =3;
				ritardo1=codify[14]+1;
                ritardo2=codify[15]+1;
				if(yes_print==1) fprintf(cfPtr, "Ritardo1= %d\n",ritardo1);
			    if(yes_print==1) fprintf(cfPtr, "Ritardo2= %d\n",ritardo2);
				value_fitness = calcule_fitness_2(which_serie_1,which_serie_2, pos_thres_1,pos_thres_2,n_regime, type_model,0, ritardo1, ritardo2);
			}
					
		}

		else { // se le componenti della soglia bivariata sono diverse
			if(yes_print==1) fprintf(cfPtr, "which_serie1= %d\n",which_serie_1);
			if(yes_print==1) fprintf(cfPtr, "which_serie2= %d\n",which_serie_2);

			
			pos_thres_1 = codify[8];
			pos_thres_2 = codify[9];
			if(yes_print==1) fprintf(cfPtr, "pos_thres_1= %d\n",pos_thres_1);
			if(yes_print==1) fprintf(cfPtr, "pos_thres_2= %d\n",pos_thres_2);
			if(codify[10]==0) {
				if(yes_print==1) fprintf(cfPtr, "Modello con 4 regimi\n");
				type_model =0;
				n_regime =4;
				ritardo1=codify[14]+1;
                ritardo2=codify[15]+1;
				if(yes_print==1) fprintf(cfPtr, "Ritardo1= %d\n",ritardo1);
			    if(yes_print==1) fprintf(cfPtr, "Ritardo2= %d\n",ritardo2);
				value_fitness = calcule_fitness_2(which_serie_1,which_serie_2, pos_thres_1,pos_thres_2,n_regime, type_model,0, ritardo1, ritardo2);
			}
			if(codify[10]==1) {
				type_model =codify[11];
				if(yes_print==1) fprintf(cfPtr, "type_model=%d\n", type_model);
				if(type_model==0) n_regime =4;
				else n_regime =3;
				ritardo1=codify[14]+1;
                ritardo2=codify[15]+1;
				if(yes_print==1) fprintf(cfPtr, "Ritardo1= %d\n",ritardo1);
			    if(yes_print==1) fprintf(cfPtr, "Ritardo2= %d\n",ritardo2);
				value_fitness = calcule_fitness_2(which_serie_1,which_serie_2, pos_thres_1,pos_thres_2,n_regime, type_model,0, ritardo1, ritardo2);
			}
		}
		
	}

    if(yes_print==1) fprintf(cfPtr," AIC = %f\n",value_fitness); 
	if(value_fitness==0.0) critter->init_fitness =0.0;
	else critter->init_fitness = exp(-(value_fitness)/100.0);
	if(yes_print==1) fprintf(cfPtr," Fitness = %f\n",exp(-(value_fitness))); 
	if(yes_print==1) fclose(cfPtr);

}

/* Questa routine ha lo scopo di calcolare la fitness di un modello MSETAR con una soglia univariata. 
   I parametri di ingresso sono: la componente della serie multivariata che rappresenta la soglia, il
   numero di regimi e le posizioni delle soglie.

*/



double calcule_fitness_1(int which_serie, int n_regime, int *pos_threshold, int stampa, int ritardo) {

	// Dichiarazione di variabili asuliare
	int i,j, ii, h, k, p, q, mm, nn, lag_d, t_d, t, jj, maxint;
	int i_reg1=0, i_reg2=0, i_reg3=0, i_reg4=0, index_var;
	int n_oss_reg1=0,n_oss_reg2=0,n_oss_reg3=0,n_oss_reg4=0; // # osservazioni nei regimi
	double sum, aic_total, aic[4]={0.0, 0.0, 0.0, 0.0};
	double R[4]={1000000.0,1000000.0,1000000.0,1000000.0}; //  R[]  il vettore delle soglie
	double R_ord[4]={1000000.0,1000000.0,1000000.0,1000000.0};
	int i1, time;
	int yes_print;
	static int chiamata=0;
	FILE *cfPtr1, *cfprev1, *cfres1;

	lag_d=ritardo;

	// Dichiarazioni di variabili riferite al problema della stima del MULTISETAR
	int n_oss, n_comp, m, x_flag;
	n_oss = number_oss;
	n_comp = number_comp;
	m=order_VAR;
	yes_print = stampa;
	

	chiamata = chiamata + 1;

	// Dichiarazioni dei vettori e matrici
    double **serie_storica, *y, **Z;
	double **Z_trasp, **Z_Z, **iZZ_Z, **C, **matrix_id, **D2; // sono matrici asuliare 
	double *AA, **FIT, **Residual,**Sigma, *medie, **FIT2, **Residual2;  // Vettore che memorizza i coefficienti
	int **indici, *vector_time;

	// dichiarazioni per il calcolo del determinante
	int *indx;
	double pp;



	// Allocazione dinamica di vettori e matrici
	serie_storica=dmatrix(1,n_comp, 1, n_oss);
	y = dvector(1,n_oss*n_comp); Z=dmatrix(1,m*n_comp,1,n_oss);
	Z_trasp=dmatrix(1,n_oss, 1, m*n_comp); Z_Z=dmatrix(1,m*n_comp,1,m*n_comp); iZZ_Z = dmatrix(1,m*n_comp,1,n_oss);
	C = dmatrix(1,(m*n_comp)*n_comp,1,n_oss*n_comp);
	AA = dvector (1,m*n_comp*n_comp);
	matrix_id = dmatrix(1,n_comp,1,n_comp);
	indici = imatrix(1,4,1,n_oss);
	vector_time=ivector(1,n_oss);
	Sigma = dmatrix(1,n_comp,1,n_comp);
	medie=dvector(1,n_comp);
	FIT2 = dmatrix(1,n_oss,1,n_comp);
	Residual2 = dmatrix(1,n_oss,1,n_comp);



	//********************* Creazione di una matrice di identit n_comp x n_comp
	for(i=1; i<=n_comp; i++) {
		for(j=1; j<=n_comp; j++) {
			if(i==j) 
				matrix_id[i][j]=1.0;
			else
				matrix_id[i][j]=0.0;
		}
	}



	//  Apro i file per la scrittura dei risultati
	if(yes_print==1) 
		if((cfPtr1  = fopen ("OutputThresholdUnivariate.txt", "a")) ==NULL) printf("File could not be opened\n");
	if(yes_print==1)		
		if((cfprev1 = fopen ("PredictionThresholdUnivariate.txt", "a")) ==NULL) printf("File could not be opened\n");
    if(yes_print==1)
		if((cfres1  = fopen ("ResidualThresholdUnivariate.txt", "a")) ==NULL) printf("File could not be opened\n");
	
	
	if(yes_print==1) fprintf(cfPtr1, "Replication of GA:%d\n", number_rep);
	if(yes_print==1) fprintf(cfPtr1, "Number of Regime = %d \n",n_regime);
	if(yes_print==1) fprintf(cfPtr1, "Threshold Variable: Component= %d\t Lag =%d\n",which_serie, lag_d);
	

	// ******************* Creazione della matrice Trasposta ***********************
	for(i=1; i<=n_comp; i++) {
		for(j=1; j<=n_oss; j++) {
			serie_storica[i][j] = serie[j][i];
			//fprintf(cfPtr, "serie_storica[%d][%d] = %.4f\n", i, j, serie_storica[i][j]);
		}
	}


    // Stabiliamo il valore delle soglie	
	for(i=0; i<n_regime-1; i++) {
		if(pos_threshold[i]==0) pos_threshold[i]=30;
		R[i] = serie[pos_threshold[i]][which_serie];
		
	}

	// Qui serve un ordinamento delle soglie
	double big = 100000000.0, amin;
	int j_aus;
	for(k=0; k<n_regime-1; k++) { 
		amin = big;
		for(i=0; i<n_regime-1; i++) {
			if(R[i] >= amin)
				continue;
			else {
			amin = R[i];
			j_aus= i;
			}
			
		} 
		R_ord[k] = R[j_aus]; 
		R[j_aus]= big;
	} // chiusura dell'ordinamento 

	for(i=0; i<n_regime-1; i++) {
		R[i] =R_ord[i]; 
	}

	/*if(yes_print==1) {
		fprintf(cfPtr1, "Thresholds:");
		for(i=0; i<n_regime-1; i++) {
			fprintf(cfPtr1, "%.6f\t",R[i]);
		}
		fprintf(cfPtr1, "\n");
	}*/

	

   // *************************  Calcolo della variabile  indicatrice di regime: indici[i][j] *********************

    // Inizializzo la matrice 
	for(ii=1; ii<=n_regime; ii++) {
		for(i=1; i<=n_oss; i++) indici[ii][i] = 0;
	}


	for(index_var=1; index_var<=order_VAR; index_var++) {
    m=index_var;

	if (lag_d>m) maxint=lag_d;
	else maxint=m;
    for(t=maxint+1; t<=n_oss; t++) {
		t_d= t-lag_d;
		if(serie[t_d][which_serie] <= R[0]) { // condizione primo regime
			ii=1;
			indici[ii][t] =  1;
			n_oss_reg1 = n_oss_reg1 +1;
		}
		if(serie[t_d][which_serie] > R[0] && serie[t_d][which_serie] <= R[1]) { // condizione secondo regime
			ii=2;
			indici[ii][t] =  1;
			n_oss_reg2 = n_oss_reg2 +1;
		}
		if(serie[t_d][which_serie] > R[1] && serie[t_d][which_serie] <= R[2]) { // condizione terzo regime
			ii=3;
			indici[ii][t] =  1;
			n_oss_reg3 = n_oss_reg3 +1;
		}
		if(serie[t_d][which_serie] > R[2]) { // condizione quarto regime
			ii=4;
			indici[ii][t] =  1;
			n_oss_reg4 = n_oss_reg4 +1;
		}
	}


    /*if(yes_print==1) {
		fprintf(cfPtr1, "ossreg1= %d  ossreg2= %d  ossreg3= %d  ossreg4= %d\n",n_oss_reg1,n_oss_reg2,n_oss_reg3,n_oss_reg4);
	}*/




/*  Per stimare i coefficienti di un modello VAR mediante il metodo MLS occorre utilizzare la seguente formula:

                                a=(kron(((inv(Z*Z'))*Z),eye(S0.n_series)))*y;

	dove a sono i parametri stimati, Z  matrice dei regressori e y  la variabile dipendente
*/
	// occorre costruire un vettore colonna y[] di dimensione (n_oss*n_comp)x1 in cui il primo elemento  la prima oss. 
	// della prima serie,il secondo  la prima oss. della seconda serie, il terzo  la seconda oss. della prima serie e cos via
	
	

	// ***************************** Costruzione di un multisetar **********************************

	// Costruzione della matrice dei regressori Z[m*n_comp][n_oss]
	for(i=1; i<=(m*n_comp); i++) {
		for(j=1; j<=n_oss; j++) {
			Z[i][j]= 0.0;
		}
	}

	
	for(i=1; i<=(m*n_comp); i++) {
		for(j=1; j<=n_oss; j++) {
			Z_trasp[j][i] = 0.0;
		}
	}


	// Stimiamo il setar regime per regime
	int i_1=0;
	for(ii=1; ii<=n_regime; ii++) {
		i_reg1=0;
		i1=0;
		for(t=m+1; t<=n_oss; t++) {
			if(indici[ii][t] ==  1) { // condizione primo regime
				i_reg1 = i_reg1 +1;
				vector_time[i_reg1]=t;
				for(j=1; j<=n_comp; j++) {
					y[i1+j] = serie[t][j];
				}
				i1 = i1+n_comp;
				// Costruzione della matrice dei regressori Z[m*n_comp][n_oss]
				for(h=1; h<=m; h++) {
					for(i=1;i<=n_comp; i++) {
						Z[(h-1)*n_comp+i][i_reg1] = serie_storica[i][t-h];
					}
				}
			}
			else continue;
		}
   
		if(yes_print==1) {
			fprintf(cfPtr1, "\nRegime: %d\n",ii);
			fprintf(cfprev1, "\nRegime: %d\n",ii);
			fprintf(cfres1, "\nRegime: %d\n",ii);
			if(ii<n_regime) fprintf(cfPtr1, "Threshold = %f\n",R[ii-1]);
		}
		if(yes_print==1) {
			if(ii==1) fprintf(cfPtr1, "Number of Observation = %d\n",n_oss_reg1);
			if(ii==2) fprintf(cfPtr1, "Number of Observation = %d\n",n_oss_reg2);
			if(ii==3) fprintf(cfPtr1, "Number of Observation = %d\n",n_oss_reg3);
			if(ii==4) fprintf(cfPtr1, "Number of Observation = %d\n",n_oss_reg4);
		}


		// ***************************** Se non vi sono abbastanza osservazioni*************
		if(i_reg1<oss_min) {
			
			//if(yes_print==2) fprintf(cfPtr1, "numero insufficiente di osservazioni:regime %d\n", ii);
	
			// Libero tutta la memoria
			free_dmatrix(serie_storica,1,n_comp, 1, n_oss);
			free_dvector(y,1,n_oss*n_comp);
			free_dmatrix(Z,1,m*n_comp,1,n_oss);
			free_dmatrix(Z_trasp,1,n_oss, 1, m*n_comp);
			free_dmatrix(Z_Z,1,m*n_comp,1,m*n_comp); 
			free_dmatrix(iZZ_Z,1,m*n_comp,1,n_oss);
			free_dmatrix(C,1,(m*n_comp)*n_comp,1,n_oss*n_comp);
			free_dvector (AA,1,m*n_comp*n_comp);
			free_dmatrix(matrix_id,1,n_comp,1,n_comp);
			free_imatrix(indici,1,4,1,n_oss);
			free_ivector(vector_time,1,n_oss);
			free_dmatrix(Sigma,1,n_comp,1,n_comp);
			free_dvector(medie,1,n_comp);
			free_dmatrix(FIT2,1,n_oss,1,n_comp);
	        free_dmatrix(Residual2,1,n_oss,1,n_comp);


			// Chiudo il file di testo
			if(yes_print==1) {
				fclose(cfPtr1);
				fclose(cfprev1);
				fclose(cfres1);
			}

			// restituisco una fitness molto piccola
			return aic_total=0.0;

		}


		// Se nel regime ii-esimo c sono abbastanza osservazioni allochiamo la memoria
		// per il calcolo dei valori fittati e dei residui
		if (i_reg1 >1) {
			FIT = dmatrix(1,i_reg1,1,n_comp);
			Residual = dmatrix(1,i_reg1,1,n_comp);
		}


        // Calcolo la trasposta di Z
		for(i=1; i<=(m*n_comp); i++) {
			for(j=1; j<=i_reg1; j++) {
				Z_trasp[j][i] = Z[i][j];
			}
		}

		
		// ********Calcolo Z*Z'*********
		for(i=1; i<=(m*n_comp); i++) {
			for(j=1; j<=(m*n_comp); j++) {
				sum = 0.0;
				for(k=1; k<=i_reg1; k++) {
					sum = sum + Z[i][k]*Z_trasp[k][j];
				}
				Z_Z[i][j] = sum;
			}
		}
        
		// Calcolo dell'inversa di Z*Z'
		x_flag = gaussj(Z_Z,(m*n_comp),(m*n_comp));

  
		// ***************Calcolo di (inv(Z*Z'))*Z*************
		for(i=1; i<=(m*n_comp); i++) {
			for(j=1; j<=i_reg1; j++) {
				sum=0.0;
				for(k=1; k<=(m*n_comp); k++) {
					sum = sum + Z_Z[i][k]*Z[k][j];
				}
				iZZ_Z[i][j] = sum;
			}
		}

		// Calcolo di kron(((inv(Z*Z'))*Z),eye(S0.n_series))
		p = n_comp;
		q = n_comp;
		mm = (m*n_comp);
		nn = i_reg1;
		for(i=1; i<=(m*n_comp); i++) {
			for(j=1; j<=n_comp; j++) {
				for(k=1; k<=i_reg1; k++) {
					for(h=1; h<=n_comp; h++) {
						C[(i-1)*p+j][(k-1)*q +h] = iZZ_Z[i][k]*matrix_id[j][h];
					}
				}
			}
		}

		// Calcolo di (kron(((inv(Z*Z'))*Z),eye(S0.n_series)))*y
		for(i=1; i<=(mm*p); i++) {
			sum=0.0;
			for(k=1; k<=(nn*q); k++) {
				sum = sum + C[i][k]*y[k];
			}
			AA[i] = sum;
			if(yes_print==1) fprintf(cfPtr1, "Phi[%d]=%.5f\n", i, AA[i]);
		}

    	// Calcolo dei valori fittati per ciascuna componente
		for(j=1; j<=n_comp; j++) {
			for(i=1; i<=i_reg1;i++){
				sum =0;
				time = vector_time[i];
				for(h=1; h<=m; h++) {
					for(k=1;k<=n_comp; k++) {
						jj = 1+(h-1)*(n_comp*n_comp) +(k-1)*n_comp+(j-1);
						sum = sum + AA[jj]*serie[time-h][k];
					}
				}
				FIT[i][j]=sum;
				
			}
		}

		if(yes_print==1) {
			for(i=1; i<=i_reg1;i++){
				for(j=1; j<=n_comp; j++) {
					fprintf(cfprev1, "FIT[%d][%d]= %f\t", i,j,FIT[i][j]);
				}
				fprintf(cfprev1, "\n");
			}
		}

		// ******************Calcolo dei residui di regime *************************+
		for(i=1; i<=i_reg1;i++){
			for(j=1; j<=n_comp; j++) {
				Residual[i][j] = serie[vector_time[i]][j]-FIT[i][j];
			}
		}
		if(yes_print==1) { 
			for(i=1; i<=i_reg1;i++){
				for(j=1; j<=n_comp; j++) {
					fprintf(cfres1, "Residual[%d][%d]= %.12f\t", i,j,Residual[i][j]);
				}
				fprintf(cfres1, "\n");
			}
		}

		//if(yes_print==1) fprintf(cfPtr, "regim = %d   i_1 = %d  i_reg1=%d\n", ii, i_1, i_reg1);
		//if(yes_print==1) fprintf(cfPtr, "regim = %d   i_1 = %d  i_reg1=%d\n", ii, i_1, i_reg1);

		
		// ******************Calcolo dei residui totali *************************
		for(i=1; i<=i_reg1;i++){
			for(j=1; j<=n_comp; j++) {
				Residual2[i+i_1][j] = Residual[i][j];
			}
		}

		// Stima della matrice di varianze e covarianze

		// Calcolo delle medie
		for(j=1; j<=n_comp; j++) {
			sum =0.0;
			for(h=1; h<=i_reg1; h++) sum = sum + Residual[h][j];
			medie[j] = sum/(double)i_reg1;;
		}

		for(j=1; j<=n_comp; j++) {
				for(i=1; i<=n_comp; i++) {
					sum =0;
					for(h=1; h<=i_reg1; h++) {
						sum = sum + (Residual[h][j]-medie[j])*(Residual[h][i]-medie[i]);
					}
					Sigma[j][i]=sum/(double)i_reg1;
					if(yes_print==1) fprintf(cfPtr1, "Sigma[%d][%d] =%.10f\n", j,i,Sigma[j][i]);
				}
		}

		// Calcolo del determinante
		indx=ivector(1,n_comp);
		D2 = dmatrix(1,n_comp,1,n_comp);
		for(i=1; i<=n_comp; i++) {
			for(j=1; j<=n_comp; j++) {
				D2[i][j] = Sigma[i][j];
			}
		}
		
		ludcmp(D2,n_comp,indx,&pp);


		double deter=1.0;
		for(j=1; j<=n_comp; j++)  deter= deter * D2[j][j];
		deter = pp*deter;
		
		// Se il determinanate  maggiore di zero
		if(deter>0.0) {

			// 1 Libero la memoria
			free_dmatrix(D2, 1,n_comp,1,n_comp);
			free_ivector(indx,1,n_oss);
			free_dmatrix(FIT,1,i_reg1,1,n_comp);
			free_dmatrix(Residual,1,i_reg1,1,n_comp);

			// calcolo dell'AIC per ii-esimo regime
			aic[ii-1] = (double)i_reg1*(log(deter)) + (2.0/(double)i_reg1)*(m)*(n_comp*n_comp);
			//if(yes_print==1) fprintf(cfPtr1, "AIC[%d]=%f\n", ii,aic[ii-1]);
		}
		else {
			free_dmatrix(D2, 1,n_comp,1,n_comp);
			free_ivector(indx,1,n_oss);
			free_dmatrix(FIT,1,i_reg1,1,n_comp);
			free_dmatrix(Residual,1,i_reg1,1,n_comp);
			aic[ii-1]=-10000.000;
		}
		i_1= i_1+i_reg1;


	} // chiusura del ciclo del regime ii
	}



	/*if(yes_print==2) { 
		for(i=1; i<=n_oss;i++){
			for(j=1; j<=n_comp; j++) {
				fprintf(cfPtr1, "Residual2[%d][%d]= %.12f\t", i,j,Residual2[i][j]);
			}
			fprintf(cfPtr1, "\n");
		}
	}*/

    // Stima della matrice di varianze e covarianze dei residui totali

	// Calcolo delle medie
	for(j=1; j<=n_comp; j++) {
		sum =0.0;
		for(h=1; h<=n_oss; h++) sum = sum + Residual2[h][j];
		medie[j] = sum/(double)n_oss;;
	}

	if(yes_print==1) fprintf(cfPtr1, "\nResidual VAriance-Covarince Matrix:\n");

	for(j=1; j<=n_comp; j++) {
		for(i=1; i<=n_comp; i++) {
			sum =0;
			for(h=1; h<=n_oss; h++) {
				sum = sum + (Residual2[h][j]-medie[j])*(Residual2[h][i]-medie[i]);
			}
			Sigma[j][i]=sum/(double)n_oss;
			if(yes_print==1) {
				fprintf(cfPtr1, "Sigma[%d][%d] =%.10f\n", j,i,Sigma[j][i]);
			}
		}
	}


	indx=ivector(1,n_comp);
	D2 = dmatrix(1,n_comp,1,n_comp);
	for(i=1; i<=n_comp; i++) {
		for(j=1; j<=n_comp; j++) {
			D2[i][j] = Sigma[i][j];
		}
	}
		
	ludcmp(D2,n_comp,indx,&pp);


	double deter=1.0;
	for(j=1; j<=n_comp; j++)  deter= deter * D2[j][j];
	deter = pp*deter;
		
	double aic_global;
	int n_tot_par = n_regime * (order_VAR)*(n_comp*n_comp);
	aic_global= (log(deter)) + (2.0/(double)n_oss)*n_tot_par;
	if(yes_print==1) fprintf(cfPtr1,"AIC =%f\n",aic_global);
//	if(yes_print==1) fprintf(cfPtr1," Fit2 = %f\n",exp(-(aic_global)/100.0));


	aic_total=0.0;
	for(ii=1; ii<=n_regime;ii++) aic_total = aic_total + aic[ii-1];
	aic_total = aic_total/(double) n_oss;
//	if(yes_print==2) fprintf(cfPtr1,"aic_total=%f\n",aic_total);


/*	if(aic_total != 0.0) {
		if(yes_print==1) fprintf(cfPtr1," aic_total = %f  Fitness = %f\n",aic_total, exp(-(aic_total)/100.0));
	}*/


	// Libero tutta la memoria
	free_dmatrix(serie_storica,1,n_comp, 1, n_oss);
	free_dvector(y,1,n_oss*n_comp);
	free_dmatrix(Z,1,m*n_comp,1,n_oss);
	free_dmatrix(Z_trasp,1,n_oss, 1, m*n_comp);
	free_dmatrix(Z_Z,1,m*n_comp,1,m*n_comp); 
	free_dmatrix(iZZ_Z,1,m*n_comp,1,n_oss);
	free_dmatrix(C,1,(m*n_comp)*n_comp,1,n_oss*n_comp);
	free_dvector (AA,1,m*n_comp*n_comp);
	free_dmatrix(matrix_id,1,n_comp,1,n_comp);
	free_imatrix(indici,1,4,1,n_oss);
	free_ivector(vector_time,1,n_oss);
	free_dmatrix(Sigma,1,n_comp,1,n_comp);
	free_dvector(medie,1,n_comp);
	free_dmatrix(FIT2,1,n_oss,1,n_comp);
	free_dmatrix(Residual2,1,n_oss,1,n_comp);
	free_dmatrix(D2, 1,n_comp,1,n_comp);
	free_ivector(indx,1,n_oss);



	if(yes_print==1) {
		fclose(cfPtr1);
		fclose(cfprev1);
		fclose(cfres1);
	}
	//return aic_total;
	return aic_global;
	

	
} // Chiusura calcule_fitness()





double calcule_fitness_2(int which_serie_1, int which_serie_2, int pos_thres_1, int pos_thres_2, int n_regime, int type_model, int stampa, int ritardo1, int ritardo2) {

	// Dichiarazione di variabili asuliare
	int i,j, ii, h, k, p, q, mm, nn, lag_d1, lag_d2, t_d1, t_d2, t, jj, maxrit, maxint;
	int i_reg1=0, i_reg2=0, i_reg3=0, i_reg4=0, index_var;
	double sum, aic_total, aic[4]={0.0, 0.0, 0.0, 0.0};;
	double R[2];
	int i1, yes_print, time;
	FILE *cfPtr, *cfprev, *cfres;

	lag_d1=ritardo1;
	lag_d2=ritardo2;

	// Dichiarazioni di variabili riferite al problema
	int n_oss, n_comp, m, x_flag;
	n_oss = number_oss;
	n_comp = number_comp;
	m=order_VAR;
	yes_print = stampa;

	// dichiarazioni per il calcolo del determinante
	int *indx;
	double pp;


	// Dichiarazioni dei vettori e matrici
    double **serie_storica, *y, **Z;
	double **Z_trasp, **Z_Z, **iZZ_Z, **C, **matrix_id, **D2; // sono matrici ausiliarie 
	double *AA, **Sigma, **FIT, **Residual, *medie, **FIT2, **Residual2;  // Vettore che memorizza i coefficienti
	int **indici, *vector_time;


	// Allocazione dinamica di vettori e matrici
	serie_storica=dmatrix(1,n_comp, 1, n_oss);
	y = dvector(1,n_oss*n_comp); Z=dmatrix(1,m*n_comp,1,n_oss);
	Z_trasp=dmatrix(1,n_oss, 1, m*n_comp); Z_Z=dmatrix(1,m*n_comp,1,m*n_comp); iZZ_Z = dmatrix(1,m*n_comp,1,n_oss);
	C = dmatrix(1,(m*n_comp)*n_comp,1,n_oss*n_comp);
	AA = dvector (1,m*n_comp*n_comp);
	matrix_id = dmatrix(1,n_comp,1,n_comp);
	indici = imatrix(1,4,1,n_oss);
	vector_time=ivector(1,n_oss);
	Sigma = dmatrix(1,n_comp,1,n_comp);
	medie=dvector(1,n_comp);
	FIT2 = dmatrix(1,n_oss,1,n_comp);
	Residual2 = dmatrix(1,n_oss,1,n_comp);



	// Creazione di una matrice di identit n_comp x n_comp
	for(i=1; i<=n_comp; i++) {
		for(j=1; j<=n_comp; j++) {
			if(i==j) 
				matrix_id[i][j]=1.0;
			else
				matrix_id[i][j]=0.0;
		}
	}

	//  Apro il file di nome Output.txt per la scrittura dei risultati
	if(yes_print==2) { 
		if((cfPtr = fopen ("OutputFitness2.txt", "a")) ==NULL) printf("File could not be opened\n");
		fprintf(cfPtr, " n_oss = %d n_comp=%d  m=%d\n",n_oss,n_comp, m);
		fprintf(cfPtr, " n_regime = %d type_model=%d\n",n_regime,type_model);
		fprintf(cfPtr, " pos_thres_1 = %d pos_thres_2=%d\n",pos_thres_1,pos_thres_2);
		fprintf(cfPtr, " which_serie_1 = %d which_serie_2=%d\n",which_serie_1,which_serie_2);
	}
	
	if(yes_print==1) if((cfPtr  = fopen ("OutputThresholdBivariate.txt", "a")) ==NULL) printf("File could not be opened\n");
	if(yes_print==1) if((cfprev = fopen ("PredictionThresholdBivariate.txt", "a")) ==NULL) printf("File could not be opened\n");
    if(yes_print==1) if((cfres  = fopen ("ResidualThresholdBivariate.txt", "a")) ==NULL) printf("File could not be opened\n");
	if(yes_print==1) fprintf(cfPtr, "Number of Regime = %d\t Type of model=%d\n",n_regime,type_model);
	if(yes_print==1) fprintf(cfPtr, "Threshold Variable: [(%d,%d) ;  (%d,%d)]\n",which_serie_1,which_serie_2,lag_d1, lag_d2);



	// ************************ Creazione della serie trasposta *************************************
	for(i=1; i<=n_comp; i++) {
		for(j=1; j<=n_oss; j++) {
			serie_storica[i][j] = serie[j][i];
			//fprintf(cfPtr, "serie_storica[%d][%d] = %.4f\n", i, j, serie_storica[i][j]);
		}
	}


	// **************** Stabiliamo il valore delle soglie *******************
	if(pos_thres_1==0) pos_thres_1=1; // Valore non valido visto che serie  stato allocato come 1,n_oss
	if(pos_thres_2==0) pos_thres_2=1; // Valore non valido visto che serie  stato allocato come 1,n_oss
	R[0] = serie[pos_thres_1][which_serie_1];
	R[1] = serie[pos_thres_2][which_serie_2];

	/*if(yes_print==1) R[0]=0.0;
	if(yes_print==1) R[1]=0.0;*/

	if(yes_print==1) fprintf(cfPtr, "Threshold 1 = %f\n",R[0]);
	if(yes_print==1) fprintf(cfPtr, "Threshold 2 = %f\n",R[1]);


	
	// **** Stabiliamo il numero di osservazioni per regime calcolando gli indicatori di regime: indici[i][j]**********

    // Inizializzo la matrice 
	for(ii=1; ii<=n_regime; ii++) {
		for(i=1; i<=n_oss; i++) indici[ii][i] = 0;
	}

	/* Regime 1: Z1>r1 Z2<r2
	   Regime 2: Z1>r1 Z2>r2
	   Regime 3: Z1<r1 Z2<r2
	   regime 4: Z1<r1 Z2>r2

    */


	// Ciclo ordine VAR

	for(index_var=1; index_var<=order_VAR; index_var++) {
		
	m= index_var;

	if(lag_d1 > lag_d2) maxrit=lag_d1;
	else maxrit= lag_d2;

	if(maxrit > m) maxint = maxrit;
	else maxint = m;

    for(t=maxrit+1; t<=n_oss; t++) {
		t_d1= t-lag_d1;
		t_d2= t-lag_d2;
		if((serie[t_d1][which_serie_1] > R[0]) && (serie[t_d2][which_serie_2] <= R[1]) ) { // condizione primo regime
			ii=1;
			if(n_regime <4 && type_model ==1) ii=1;
			if(n_regime <4 && type_model ==2) ii=1;
			if(n_regime <4 && type_model ==3) ii=3;
			if(n_regime <4 && type_model ==4) ii=1;
			if(n_regime <4 && type_model ==5) ii=1;
			if(n_regime <4 && type_model ==6) ii=1;
			indici[ii][t] =  1;
		}
		if((serie[t_d1][which_serie_1] > R[0]) && (serie[t_d2][which_serie_2] > R[1])) { // condizione secondo regime
			ii=2;
			if(n_regime <4 && type_model ==1) ii=1;
			if(n_regime <4 && type_model ==2) ii=2;
			if(n_regime <4 && type_model ==3) ii=1;
			if(n_regime <4 && type_model ==4) ii=3;
			if(n_regime <4 && type_model ==5) ii=2;
			if(n_regime <4 && type_model ==6) ii=3;

			indici[ii][t] =  1;
		}
		if((serie[t_d1][which_serie_1] <= R[0]) && (serie[t_d2][which_serie_2] <= R[1])) { // condizione terzo regime
			ii=3;
			if(n_regime <4 && type_model ==1) ii=2;
			if(n_regime <4 && type_model ==2) ii=3;
			if(n_regime <4 && type_model ==3) ii=3;
			if(n_regime <4 && type_model ==4) ii=2;
			if(n_regime <4 && type_model ==5) ii=3;
			if(n_regime <4 && type_model ==6) ii=3;

			indici[ii][t] =  1;
		}
		if((serie[t_d1][which_serie_1] <= R[0]) && (serie[t_d2][which_serie_2] > R[1])) { // condizione quarto regime
			ii=4;
			if(n_regime <4 && type_model ==1) ii=3;
			if(n_regime <4 && type_model ==2) ii=3;
			if(n_regime <4 && type_model ==3) ii=2;
			if(n_regime <4 && type_model ==4) ii=3;
			if(n_regime <4 && type_model ==5) ii=1;
			if(n_regime <4 && type_model ==6) ii=2;

			indici[ii][t] =  1;
		}
	}

	if(yes_print==2) {
		for(i=1; i<=n_oss; i++) { 
			for(ii=1; ii<=n_regime; ii++) {
				fprintf(cfPtr, "indici[%d][%d] = %d\t", ii, i, indici[ii][i]);
			}
			fprintf(cfPtr,"\n");
		}
	}

/*  Per stimare i coefficienti di un modello VAR mediante il LS occorre utilizzare la seguente formula:

                                a=(kron(((inv(Z*Z'))*Z),eye(S0.n_series)))*y;

	dobve a i parametri stimati, Z  matrice dei regressori e y  la variabile dipendente
*/
	// occorre costruire un vettore colonna y[] di dimensione (n_oss*n_comp)x1 in cui il primo elemento  la prima oss. 
	// della prima serie,il secondo  la prima oss. della seconda serie, il terzo  la seconda oss. della prima serie e cos via
	
	

	// Costruzione di un multisetar con due regimi

	// Costruzione della matrice dei regressori Z[m*n_comp][n_oss]
	for(i=1; i<=(m*n_comp); i++) {
		for(j=1; j<=n_oss; j++) {
			Z[i][j]= 0.0;
		}
	}

	
	for(i=1; i<=(m*n_comp); i++) {
		for(j=1; j<=n_oss; j++) {
			Z_trasp[j][i] = 0.0;
		}
	}


	// Stimiamo il setar regime per regime
	int i_1=0;
	for(ii=1; ii<=n_regime; ii++) {
		i_reg1=0;
		i1=0;
		for(t=m+1; t<=n_oss; t++) {
			if(indici[ii][t] ==  1) { // condizione primo regime
				i_reg1 = i_reg1 +1;
				vector_time[i_reg1]=t;
				for(j=1; j<=n_comp; j++) {
					y[i1+j] = serie[t][j];
				}
				i1 = i1+n_comp;
				// Costruzione della matrice dei regressori Z[m*n_comp][n_oss]
				for(h=1; h<=m; h++) {
					for(i=1;i<=n_comp; i++) {
						Z[(h-1)*n_comp+i][i_reg1] = serie_storica[i][t-h];
					}
				}
			}
			else continue;
		}
		if(yes_print==1) fprintf(cfPtr, "Regime: %d\n", ii);
		if(yes_print==1) fprintf(cfPtr, "Number of Observation: %d\n", i_reg1);



		// ***************************** Se non vi sono abbastanza osservazioni ****************
		if(i_reg1<oss_min) {

			if(yes_print==2) fprintf(cfPtr, "numero insufficiente di osservazioni:regime %d\n", ii);
			
			// Libero tutta la memoria
			free_dmatrix(serie_storica,1,n_comp, 1, n_oss);
			free_dvector(y,1,n_oss*n_comp);
			free_dmatrix(Z,1,m*n_comp,1,n_oss);
			free_dmatrix(Z_trasp,1,n_oss, 1, m*n_comp);
			free_dmatrix(Z_Z,1,m*n_comp,1,m*n_comp); 
			free_dmatrix(iZZ_Z,1,m*n_comp,1,n_oss);
			free_dmatrix(C,1,(m*n_comp)*n_comp,1,n_oss*n_comp);
			free_dvector (AA,1,m*n_comp*n_comp);
			free_dmatrix(matrix_id,1,n_comp,1,n_comp);
			free_imatrix(indici,1,4,1,n_oss);
			free_ivector(vector_time,1,n_oss);
			free_dmatrix(Sigma,1,n_comp,1,n_comp);
			free_dvector(medie,1,n_comp);
			free_dmatrix(FIT2,1,n_oss,1,n_comp);
	        free_dmatrix(Residual2,1,n_oss,1,n_comp);


			// Chiudo il file di testo
			if(yes_print==1) {
				fclose(cfPtr);
				fclose(cfprev);
				fclose(cfres);
			}

			// Restituisco un valore di fitness molto piccolo
			return aic_total=0.0;
		}

        
		if (i_reg1 >1) {
			FIT = dmatrix(1,i_reg1,1,n_comp);
			Residual = dmatrix(1,i_reg1,1,n_comp);
		}

		if(yes_print==2) {
			for(i=1; i<=(m*n_comp); i++) {
				for(j=1; j<=10; j++) {
					fprintf(cfPtr, "Z[%d][%d] = %.4f\t", i,j, Z[i][j]);
				}
			fprintf(cfPtr, "\n");
			}
		}

        // Calcolo la trasposta di Z
		for(i=1; i<=(m*n_comp); i++) {
			for(j=1; j<=i_reg1; j++) {
				Z_trasp[j][i] = Z[i][j];
			}
		}

		// Calcolo Z*Z'
		for(i=1; i<=(m*n_comp); i++) {
			for(j=1; j<=(m*n_comp); j++) {
				sum = 0.0;
				for(k=1; k<=i_reg1; k++) {
					sum = sum + Z[i][k]*Z_trasp[k][j];
				}
				Z_Z[i][j] = sum;
			}
		}

		if(yes_print==2) {
			for(i=1; i<=(m*n_comp); i++) {
				for(j=1; j<=(m*n_comp); j++) {
					fprintf(cfPtr, "Z_Z[%d][%d] = %.5f\t", i,j,Z_Z[i][j]);
				}
				fprintf(cfPtr, "\n");
			}
		}


		// Calcolo dell'inversa di Z*Z'
		x_flag = gaussj(Z_Z,(m*n_comp),(m*n_comp));

		if(yes_print==2) {
			fprintf(cfPtr, "Inversa di Z*Z'\n");
			for(i=1; i<=(m*n_comp); i++) {
				for(j=1; j<=(m*n_comp); j++) {
					fprintf(cfPtr, "%.5f\t", Z_Z[i][j]);
				}
				fprintf(cfPtr, "\n");
			}
		}
    
		// Calcolo di (inv(Z*Z'))*Z
		for(i=1; i<=(m*n_comp); i++) {
			for(j=1; j<=i_reg1; j++) {
				sum=0.0;
				for(k=1; k<=(m*n_comp); k++) {
					sum = sum + Z_Z[i][k]*Z[k][j];
				}
				iZZ_Z[i][j] = sum;
			}
		}

	    if(yes_print==2) {
			fprintf(cfPtr, "inv(Z*Z')Z \n");
			for(i=1; i<=(m*n_comp); i++) {
				for(j=1; j<=10; j++) {
					fprintf(cfPtr, "%.5f\t",iZZ_Z[i][j]);
				}
				fprintf(cfPtr, "\n");
			}
		}

		// Calcolo di kron(((inv(Z*Z'))*Z),eye(S0.n_series))
		p = n_comp;
		q = n_comp;
		mm = (m*n_comp);
		nn = i_reg1;
		for(i=1; i<=(m*n_comp); i++) {
			for(j=1; j<=n_comp; j++) {
				for(k=1; k<=i_reg1; k++) {
					for(h=1; h<=n_comp; h++) {
						C[(i-1)*p+j][(k-1)*q +h] = iZZ_Z[i][k]*matrix_id[j][h];
					}
				}
			}
		}

		if(yes_print==2) {
			fprintf(cfPtr, "Stampa di C\n");
			for(i=1; i<=(mm*p); i++) {
				for(j=1; j<=10; j++) {
					fprintf(cfPtr, "%.4f\t", C[i][j]);
				}
				fprintf(cfPtr, "\n");
			}
		}

		
		// Calcolo di (kron(((inv(Z*Z'))*Z),eye(S0.n_series)))*y
		for(i=1; i<=(mm*p); i++) {
			sum=0.0;
			for(k=1; k<=(nn*q); k++) {
				sum = sum + C[i][k]*y[k];
			}
			AA[i] = sum;
			if(yes_print==1) fprintf(cfPtr, "Phi[%d]=%.5f\n", i, AA[i]);
		}



		// ********************************Calcolo dei valori fittati per ciascuna componente

		for(j=1; j<=n_comp; j++) {
			for(i=1; i<=i_reg1;i++){
				sum =0;
				time = vector_time[i];
				//fprintf(cfPtr, "TIME = %d\n", time);
				for(h=1; h<=m; h++) {
					for(k=1;k<=n_comp; k++) {
						jj = 1+(h-1)*(n_comp*n_comp) +(k-1)*n_comp+(j-1);
						//fprintf(cfPtr, "j=%d i=%d h=%d  k=%d  jj = %d\n", j, i, h, k, jj);
						sum = sum + AA[jj]*serie[time-h][k];
					}
				}
				FIT[i][j]=sum;
				
			}
		}

		if(yes_print==1) {
			for(i=1; i<=i_reg1;i++){
				for(j=1; j<=n_comp; j++) {
					fprintf(cfprev, "FIT[%d][%d]= %f\t", i,j,FIT[i][j]);
				}
				fprintf(cfprev, "\n");
			}
		}

		// ******************Calcolo dei residui*************************+
		for(i=1; i<=i_reg1;i++){
			for(j=1; j<=n_comp; j++) {
				Residual[i][j] = serie[vector_time[i]][j]-FIT[i][j];
			}
		}
		if(yes_print==1) { 
			for(i=1; i<=i_reg1;i++){
				for(j=1; j<=n_comp; j++) {
					fprintf(cfres, "Residual[%d][%d]= %f\t", i,j,Residual[i][j]);
				}
				fprintf(cfres, "\n");
			}
		}


		/*if(yes_print==1) { 
			for(i=1; i<=i_reg1;i++){
				for(j=1; j<=n_comp; j++) {
					fprintf(cfPtr, "Serie[%d][%d]= %f\t", vector_time[i],j,serie[vector_time[i]][j]);
				}
				fprintf(cfPtr, "\n");
			}
		}*/



		
		// ******************Calcolo dei residui totali *************************
		for(i=1; i<=i_reg1;i++){
			for(j=1; j<=n_comp; j++) {
				Residual2[i+i_1][j] = Residual[i][j];
			}
		}



		// *******************************Stima della matrice di varianze e covarianze

		// Calcolo delle medie
		for(j=1; j<=n_comp; j++) {
			sum =0.0;
			for(h=1; h<=i_reg1; h++) sum = sum + Residual[h][j];
			medie[j] = sum/(double)i_reg1;;
			if(yes_print==2) fprintf(cfPtr, "medie[%d] =%.15f\n", j,medie[j]);
		}


		for(j=1; j<=n_comp; j++) {
				for(i=1; i<=n_comp; i++) {
					sum =0;
					for(h=1; h<=i_reg1; h++) {
						sum = sum + (Residual[h][j]-medie[j])*(Residual[h][i]-medie[i]);
					}
					Sigma[j][i]=sum/(double)i_reg1;
					if(yes_print==1) fprintf(cfPtr, "sigma[%d][%d] =%.10f\n", j,i,Sigma[j][i]);
				}
		}

		// *******************************************Calcolo del determinante
		indx=ivector(1,n_comp);
		D2 = dmatrix(1,n_comp,1,n_comp);
		for(i=1; i<=n_comp; i++) {
			for(j=1; j<=n_comp; j++) {
				D2[i][j] = Sigma[i][j];
				if(yes_print==2) fprintf(cfPtr, "D2(%d)(%d)=%f\t", i, j, D2[i][j]);
			}
			if(yes_print==2) fprintf(cfPtr,"\n");
		}
		
		ludcmp(D2,n_comp,indx,&pp);

		double deter=1.0;
		for(j=1; j<=n_comp; j++)  deter= deter * D2[j][j];
		deter = pp*deter;
		
		if(yes_print==2) fprintf(cfPtr,"Determinante=%e\n",deter);


		// Se il determinanate  maggiore di zero

		if(deter>0.0) {

			// 1 Libero la memoria
			free_dmatrix(D2, 1,n_comp,1,n_comp);
			free_ivector(indx,1,n_oss);
			free_dmatrix(FIT,1,i_reg1,1,n_comp);
			free_dmatrix(Residual,1,i_reg1,1,n_comp);

			// Calcolo il valore dell'AIC per il regime ii-esimo
			aic[ii-1] = (double)i_reg1*(log(deter)) + (2.0/(double)i_reg1)*(m)*(n_comp*n_comp);
			if(yes_print==2)  fprintf(cfPtr, "AIC[%d]=%f\n", ii,aic[ii]);
		}
		else {
			free_dmatrix(D2, 1,n_comp,1,n_comp);
			free_ivector(indx,1,n_oss);
			free_dmatrix(FIT,1,i_reg1,1,n_comp);
			free_dmatrix(Residual,1,i_reg1,1,n_comp);

			// Cassegno all'AIC per il regime ii-esimo un valore molto piccolo
			aic[ii-1]=-10000.000;
		}
		i_1= i_1+i_reg1;
		

	} // ciclo del regime ii
	
    } // fine ciclo index_var

	
	
	if(yes_print==2) { 
		for(i=1; i<=n_oss;i++){
			for(j=1; j<=n_comp; j++) {
				fprintf(cfPtr, "Residual2[%d][%d]= %.12f\t", i,j,Residual2[i][j]);
			}
			fprintf(cfPtr, "\n");
		}
	}



    // Stima della matrice di varianze e covarianze dei residui totali

	// Calcolo delle medie
	for(j=1; j<=n_comp; j++) {
		sum =0.0;
		for(h=1; h<=n_oss; h++) sum = sum + Residual2[h][j];
		medie[j] = sum/(double)n_oss;;
		if(yes_print==2) fprintf(cfPtr, "medie[%d] =%.15f\n", j,medie[j]);
	}

	for(j=1; j<=n_comp; j++) {
		for(i=1; i<=n_comp; i++) {
			sum =0;
			for(h=1; h<=n_oss; h++) {
				sum = sum + (Residual2[h][j]-medie[j])*(Residual2[h][i]-medie[i]);
			}
			Sigma[j][i]=sum/(double)n_oss;
			if(yes_print==1) fprintf(cfPtr, "sigma[%d][%d] =%.10f\n", j,i,Sigma[j][i]);
		}
	}



	indx=ivector(1,n_comp);
	D2 = dmatrix(1,n_comp,1,n_comp);
	for(i=1; i<=n_comp; i++) {
		for(j=1; j<=n_comp; j++) {
			D2[i][j] = Sigma[i][j];
			if(yes_print==2) fprintf(cfPtr, "D2(%d)(%d)=%e\t", i, j, D2[i][j]);
		}
		if(yes_print==2) fprintf(cfPtr,"\n");
	}
		
	ludcmp(D2,n_comp,indx,&pp);

	if(yes_print==2) {
		for(i=1; i<=n_comp; i++) {
			for(j=1; j<=n_comp; j++) {
				fprintf(cfPtr, "D2(%d)(%d)=%e\t", i, j, D2[i][j]);
			}
			fprintf(cfPtr,"\n");
		}
		fprintf(cfPtr,"pp=%f\n",pp);

	}

	double deter=1.0;
	for(j=1; j<=n_comp; j++)  deter= deter * D2[j][j];
	deter = pp*deter;
		
	if(yes_print==2) fprintf(cfPtr,"Determinante_tot=%e\n",deter);
	double aic_global;
	int n_tot_par = n_regime * (order_VAR)*(n_comp*n_comp);
	aic_global= (log(deter)) + (2.0/(double)n_oss)*n_tot_par;
	if(yes_print==1) fprintf(cfPtr,"Num_tot_par=%d  aic_global=%f\n",n_tot_par, aic_global);
	if(yes_print==1) fprintf(cfPtr," Fit2 = %f\n",exp(-(aic_global)/100.0));


	aic_total=0.0;
	for(ii=1; ii<=n_regime;ii++) aic_total = aic_total + aic[ii-1];
	aic_total = aic_total/(double) n_oss;
	if(yes_print==2) fprintf(cfPtr,"aic_total=%f\n",aic_total);


	if(aic_total != 0.0) {
		if(yes_print==1) fprintf(cfPtr," aic_total = %f  Fitness = %f\n",aic_total, exp(-(aic_total)/100.0));
	}

	// Siamo usciti dal ciclo ii: quindi abbiamo calcolato l'aic di regime

	// Libero tutta la memoria allocata
	free_dmatrix(serie_storica,1,n_comp, 1, n_oss);
	free_dvector(y,1,n_oss*n_comp);
	free_dmatrix(Z,1,m*n_comp,1,n_oss);
	free_dmatrix(Z_trasp,1,n_oss, 1, m*n_comp);
	free_dmatrix(Z_Z,1,m*n_comp,1,m*n_comp); 
	free_dmatrix(iZZ_Z,1,m*n_comp,1,n_oss);
	free_dmatrix(C,1,(m*n_comp)*n_comp,1,n_oss*n_comp);
	free_dvector (AA,1,m*n_comp*n_comp);
	free_dmatrix(matrix_id,1,n_comp,1,n_comp);
	free_imatrix(indici,1,4,1,n_oss);
	free_ivector(vector_time,1,n_oss);
	free_dmatrix(Sigma,1,n_comp,1,n_comp);
	free_dvector(medie,1,n_comp);
	free_dmatrix(FIT2,1,n_oss,1,n_comp);
	free_dmatrix(Residual2,1,n_oss,1,n_comp);


	return aic_global;


	if(yes_print==1) {
		fclose(cfPtr);
		fclose(cfprev);
		fclose(cfres);
	}

	
} // Chiusura calcule_fitness()

/* This routine is called once when a new run or extended run is started. */
/* It must set a value for all of its global variables, although they may */
/* be modified later (in other app_ functions) during the run. */

void app_set_options() {

  avg_mut_width = 3;    /* Must set if using multimut.c.  Value is expected  */
			/* number of adjacent fields (bits, if binary) to    */
			/* change at once.  */

  stochastic = 0;       /* Must set to 0 or 1 to specify whether or not */
			/* fitness function always returns same value for a  */
			/* given chrom.-- If always same, set stochastic = 0 */
			/* to avoid extra evaluations; if changes with env't */
			/* or randomly, set = 1 */

  elitism = 1;          /* Must set to 0 or 1 to determine whether or not     */
			/* best indiv. is always preserved in next generation */
			/* (1 = yes).  */
  elitism &= !stochastic;   /* Note that elitism is reset to 0 automatically  */
			/* if user specifies stochastic flag = 1, since the   */
			/* changing fitness makes it impossible to insure that*/
			/* what was once best is still best. */

  user_supplied_initialization = 0;       /* Make it TRUE (1) if you will    */
					  /* supply code to initialize the   */
					  /* individuals in oldpop below, in */
					  /* routine app_user_init_pop(),    */
					  /* instead of using one of the     */
					  /* "standard" methods supplied with*/
					  /* GALOPPS.  */

  call_app_user_does_migration = 0;   /* Flag, if set, allows user full   */
					/* control of migration, from */
					/* app_user_does_migration(), below */

  using_inversion = 0;
			 /* Set flag to 1 (TRUE) if will use inversion to  */
			 /* alter placement of fields on chromosome, with  */
			 /* remapping of fields to "standard" positions at */
			 /* each evaluation, before objective function is  */
			 /* called.  It is set automatically if pinversion>0, */
			 /* but user may set manually here if has ever used */
			 /* inversion, so unmapping, translation, etc., will*/
			 /* continue to occur, even if pinversion now set  */
			 /* to 0.  */

  different_reps = 0 || using_inversion;
			 /* You set to 1 (TRUE) if will use different reps */
			 /* in different subpops.  If TRUE, and if the */
			 /* difference is NOT ONLY INVERSION, then YOU MUST*/
			 /* also supply code to transform ALL of the */
			 /* migrating individuals, in app_transform_migrant*/
			 /* in the skeleton below, and code below in */
			 /* app_transform_chrom_to_std to transform a */
			 /* chromosome into its STANDARD representation */
			 /* (used in the objective function objfunc().  */
			 /* Always 1 if inversion is being used.  */

  if (different_reps && !using_inversion) {
      /* User is using different representations in various subpopulations, */
      /* so must be supplying app_transform_chrom_to_std() and */
      /* app_transform_migrant().  User must also set the variables */
      /* max_chromsize_among_all_subpops and max_numfields_among_all_subpops */
      /* in the lines directly below, and delete the print and the call to */
      /* exit() which are inserted as a reminder to the user.  */ 

      fprintf(stderr, "\n*** You have selected different_reps and are not ");
      fprintf(stderr,"using inversion, \nso you MUST specify here in app_init");
      fprintf(stderr, " the maximum sizes of chromsize and numfields.***\n");
      exit(-9);
  }
  else {
      max_chromsize_among_all_subpops = -1; /* set to illegal value, as flag */
      max_numfields_among_all_subpops = -1; /* set to illegal value, as flag */
  }
}




/* Routine called ONLY IF user inputs alpha_size as < 2, meaning fields have */
/* different lengths.  In that case, user MUST provide code here to set the  */
/* array field_sizes[i], 0<=i<numfields, to the number of different codes    */
/* possible in the ith field (codes will vary from 0 to field_sizes[i] - 1). */
/* Data may be set in executable statements, or via initializers.   If the   */
/* problem uses different representations in different subpopulations, and   */
/* if that affects the field sizes, then this code must set the proper sizes */
/* for each subpopulation, using popno to tell which subpop is being         */
/* initialized.  Elsewhere, the program will also automatically use the      */
/* field_sizes to set array  field_ends, to the final bit number (in         */
/* [1, lchrom]) of each field.   */

void  app_set_field_sizes() {

  field_sizes[0] = 2;  // tipo di modello
  field_sizes[1] = 2;  // quale serie costituisce la soglia 
  field_sizes[2] = 2;  // quale serie costituisce la prima componente della soglia Multi
  field_sizes[3] = 2;  // quale serie costituisce la prima componente della soglia Multi
  field_sizes[4] = 4; // numeri di regimi per modelli con soglie univariate 
  field_sizes[5] = number_oss;  // posizione della soglia
  field_sizes[6] = number_oss;  // posizione della soglia
  field_sizes[7] = number_oss;  // posizione della soglia
  field_sizes[8] = number_oss;  // posizione della soglia
  field_sizes[9] = number_oss;  // posizione della soglia
  field_sizes[10] = 2; // unire o no due regioni 
  field_sizes[11] = 4;  
  field_sizes[12] = 4;  
  field_sizes[13] = 6;
  field_sizes[14] = 6;
  field_sizes[15] = 6;

}




// Routine per la lettura dei parametri del problema da affrontare
void app_read_prob_params() {

}



/*
void app_data(BOOL    REVISING) {

    if (REVISING)
	return;

}
*/

/* this routine should contain any application-dependent computations */
/* that should be performed before each generation. called by main()    */

void application() {

}



// Routine per l'eventuale inizializzazione di variabili dell'utente 
void app_init() {
	
	static int    firstcall = 1;

	/* Any code you want to add to app_init goes below. */

  if (firstcall) {
    firstcall = 0;
    /* Put here any code you want to do just once in a run, regardless of */
    /* how run was started. */

    return;
  }

  /* Put here any code you want run for each subpopulation at the beginning */
  /* of its execution in each cycle. */

  return;
}




// Routine per la creazione della popolazione iniziale
void app_user_init_pop(int  starting_guy_index) {
}



/* Application-dependent initial report called once by initialize() */
void app_initreport() {

    int i,j;

    for (j=0;j<popsize;j++) {
	/* make printed index agree with 'external' numbering, 1 to popsize */
	fprintf(outfp, "\nInd. %d fitness: %f, fields: ", j+1, oldpop[j].init_fitness);
	for (i=0;i<numfields;i++) {
	    fprintf(outfp, "%d ", getfield(i, numfields, oldpop[j].chrom));
	}
    }
    fprintf(outfp, "\n");

}


/* Application-dependent report, called each generation by report() */

void app_report() {



    /* Normal app-dependent end-of-generation printing done here. */
    int i;
/*
    fprintf(outfp, "\n ****Best Now: Gen. %d, Ind. %d; Eval. # %ld, Fitness= %f\n"
	    , gen, bestnow+1, newpop[bestnow].neval
	    , (float)newpop[bestnow].init_fitness);
    fprintf(outfp, " Fields: ");
    for (i=0;i<numfields;i++)
	fprintf(outfp, "%d ", getfield(i, numfields, newpop[bestnow].chrom));
    fprintf(outfp, "\n");

	fprintf(outfp, "BestNow = %d\n", bestnow);
*/

	// Valutiamo il cromosomo migliore
    objfunc_print();
	//objfunc_chorm();
}




 /* Routine to be called when other output is suppressed, to check for
  * trigger for special user-defined output, or to print desired output even
  * when running in quiet mode.  Helpful when doing many long runs.
  * User can do checking for last generation of a Manypops run, for example,
  * in case want to reset quiet to 0 for a final summary printout, etc. 
  */

void app_quiet_report() {
    int i;
    /* Below, supply any logic and printing to be used in quiet mode
     * operation, when most other normal output is suppressed. */

    /* for example, print warning ONE TIME if quiet flag is on, so user knows
     * why doesn't get any output.                                         */
    if(quiet < 3 && popno == 0 && cycle == 0 && gen == startgen)
	 fprintf(outfp, "\n Flag quiet > 0, normal output suppressed.\n");

    /* Logic below starts printing end of next-to-last generation of last
     * cycle, for final summary.  (Works for GALOPPS/Onepop (ncycles == 0) */
     /* and GALOPPS/Manypops) */
    /* To enable this logic, remove comment delimiters on next line.
    /* if((cycle == ncycles - 1 || ncycles == 0) && gen == maxgen - 2)
	    quiet = 0;  */
    fprintf(outfp, "\n Best Now: Gen. %d, Ind. %d; Eval. # %ld, Fitness= %f\n"
	    , gen, bestnow+1, newpop[bestnow].neval
	    , (float)newpop[bestnow].init_fitness);
    fprintf(outfp, " Fields: ");
    for (i=0;i<numfields;i++)
	fprintf(outfp, "%d ", getfield(i, numfields, newpop[bestnow].chrom));
    fprintf(outfp, "\n");
}



/* Optional application-dependent string printer, called by report() just */
/* after it has finished printing the chromosomes in "standard" binary format.*/

void app_print_strings() {
    int i,j;

    for (j=0;j<popsize;j++) {
	/* make printed index agree with 'external' numbering, 1 to popsize */
	fprintf(outfp, "\nInd. %d fitness: %f, fields: ", j+1
		, (float)newpop[j].init_fitness);
	for (i=0;i<numfields;i++) {
	    fprintf(outfp, "%d ", getfield(i, numfields, newpop[j].chrom));
	}
    }
    fprintf(outfp, "\n");
}


/* Application-dependent report, called by reptconv() (in report.c) */
void app_conv_rept() {

}





/* Opportunity to do any desired (application-dependent) operations at the */
/* end of each generation of each subpopulation. */

void app_generate()
{

}



/* Application-dependent statistics calculations called by statistics() at  */
/* the end of each generation.                                              */

void app_stats(struct individual *pop) {
    /* See approyrd.c for an example. */
}



/* Application-dependent callback routine user must use to tell program   */
/* the size of the utility field to malloc for each chromosome.     Used  */
/* to compute the amount of space needed per individual in the checkpoint */
/* files.                                                                 */

int GetUtilitySize() {
    /* See approyrd.c for an example. */
    return 0;
}




// Creazione di una funzione che mi permetta si stampare i parametri del miglior cromosoma

void objfunc_print(void) {

	int i, j, which_serie, n_regime, which_serie_1, which_serie_2, pos_thres_1, pos_thres_2, ritardo, ritardo1, ritardo2;
	int codify[16], type_model;
	double value_fitness, sum=0.0;
	int *pos_threshold;


	pos_threshold = ivector(0,4);


    for (i=0;i<numfields;i++) codify[i] = getfield(i, numfields, newpop[bestnow].chrom);


	// Esaminiamo  ora solo modelli con soglia bivariata
	//codify[0] = 1;
	//codify[2] = 0;
	//codify[3] = 1;
	//codify[10]=0;


	if(codify[0] == 0) { // Modelli con una soglia univariata
		which_serie = codify[1]+1;
		n_regime    =  codify[4]+1;
		if(codify[4]==0) n_regime =2;
		for (j=0;j<n_regime-1;j++){
			pos_threshold[j] = codify[5+j];
		}
		ritardo=codify[13]+1;
		value_fitness = calcule_fitness_1(which_serie,n_regime,pos_threshold,1, ritardo);
	}
	else { // Modelli con soglia bivariata
		
		which_serie_1 = codify[2]+1;
		which_serie_2 = codify[3]+1;
				
		//se le componenti della soglia bivariata sono le le stesse otteniamo un modello del tipo sopra
		if(which_serie_1 == which_serie_2) {
			/*which_serie =which_serie_1; 
			n_regime    =  codify[4]+1;
			if(yes_print==1) fprintf(cfPtr, "n_regime= %d\n",n_regime);
		    for (j=0;j<n_regime-1;j++){
				pos_threshold[j] = codify[5+j];
				if(yes_print==1) fprintf(cfPtr, "pos_threshold[%d]= %d\n",j,pos_threshold[j]);
			}
			ritardo=codify[13]+1;
			value_fitness = calcule_fitness_1(which_serie,n_regime,pos_threshold,0, ritardo);*/
			if(which_serie_1==1) which_serie_2=2;
			if(which_serie_1==2) which_serie_1=1;
			pos_thres_1 = codify[8];
			pos_thres_2 = codify[9];
			if(codify[10]==0) {
				type_model =0;
				n_regime =4;
				ritardo1=codify[14]+1;
                ritardo2=codify[15]+1;
				value_fitness = calcule_fitness_2(which_serie_1,which_serie_2, pos_thres_1,pos_thres_2,n_regime, type_model,1, ritardo1, ritardo2);
			}
			if(codify[10]==1) {
				type_model =codify[11];
				if(type_model==0) n_regime =4;
				else n_regime =3;
				ritardo1=codify[14]+1;
                ritardo2=codify[15]+1;
				value_fitness = calcule_fitness_2(which_serie_1,which_serie_2, pos_thres_1,pos_thres_2,n_regime, type_model,1, ritardo1, ritardo2);
			}
					
		}

		else { // se le componenti della soglia bivariata sono diverse
				
			pos_thres_1 = codify[8];
			pos_thres_2 = codify[9];
			if(codify[10]==0) {
				type_model =0;
				n_regime =4;
				ritardo1=codify[14]+1;
                ritardo2=codify[15]+1;
				value_fitness = calcule_fitness_2(which_serie_1,which_serie_2, pos_thres_1,pos_thres_2,n_regime, type_model,1, ritardo1, ritardo2);
			}
			if(codify[10]==1) {
				type_model =codify[11];
				if(type_model==0) n_regime =4;
				else n_regime =3;
				ritardo1=codify[14]+1;
                ritardo2=codify[15]+1;
				value_fitness = calcule_fitness_2(which_serie_1,which_serie_2, pos_thres_1,pos_thres_2,n_regime, type_model,1, ritardo1, ritardo2);
			}
		}
		
	}

    /*if(yes_print==1) fprintf(cfPtr," AIC = %f\n",value_fitness); 
	if(value_fitness==0.0) critter->init_fitness =0.0;
	else critter->init_fitness = exp(-(value_fitness));
	if(yes_print==1) fprintf(cfPtr," Fitness = %f\n",exp(-(value_fitness))); 
	if(yes_print==1) fclose(cfPtr);*/

	free_ivector(pos_threshold,0,4);


}

// Funzione per esaminare particolari modelli

void objfunc_chorm(void) {

	int j, which_serie, n_regime, which_serie_1, which_serie_2, pos_thres_1, pos_thres_2, ritardo;
	int codify[13], type_model;
	double value_fitness, sum=0.0;
	int *pos_threshold;


	pos_threshold = ivector(0,4);


	// Esaminiamo per ora solo modelli con soglia univariata
	codify[0] = 0;
	codify[1]=0;
	codify[4]=0;

	// posizione delle soglie univariate
	codify[5]=239;
	codify[6]=100;
	codify[7]=120;

	// quale serie compongono la soglia bivariate
	codify[2]=0;
	codify[3]=1;

	// posizione delle soglie multivariate
	codify[8]=100;
	codify[9]=200;

	codify[10]=0;
	codify[11]=3;
	codify[12]=0;


	if(codify[0] == 0) { // Modelli con una soglia univariata
		which_serie = codify[1]+1;
		//fprintf(outfp, "which_serie= %d\n",which_serie);
		n_regime    =  codify[4]+1;
		if(codify[4]==0) n_regime =2;
		for (j=0;j<n_regime-1;j++){
			pos_threshold[j] = codify[5+j];
			//fprintf(cfPtr, "pos_threshold[%d]= %d\n",j,pos_threshold[j]);
		}
		ritardo=codify[13]+1;
		value_fitness = calcule_fitness_1(which_serie,n_regime,pos_threshold,1, ritardo);
	}
	else { // Modelli con soglia bivariata
		
		which_serie_1 = codify[2]+1;
		which_serie_2 = codify[3]+1;
		//fprintf(cfPtr, "which_serie_1= %d\n",which_serie_1);
		//fprintf(cfPtr, "which_serie_2= %d\n",which_serie_2);
		
		//se le componenti della soglia bivariata sono le le stesse otteniamo un modello del tipo sopra
		if(which_serie_1 == which_serie_2) {
			which_serie =which_serie_1; 
			n_regime    =  codify[4]+1;
			//fprintf(cfPtr, "n_regime= %d\n",n_regime);
		    for (j=0;j<n_regime-1;j++){
				pos_threshold[j] = codify[5+j];
				//fprintf(cfPtr, "pos_threshold[%d]= %d\n",j,pos_threshold[j]);
			}
			//value_fitness = calcule_fitness_1(which_serie,n_regime,pos_threshold,1);
			
		}

		else { // se le componenti della soglia bivariata sono diverse
			pos_thres_1 = codify[8];
			pos_thres_2 = codify[9];
			//fprintf(cfPtr, "pos_thres_1= %d\n",pos_thres_1);
			//fprintf(cfPtr, "pos_thres_2= %d\n",pos_thres_2);
			if(codify[10]==0) {
				//fprintf(cfPtr, "Modello con 4 regimi\n");
				type_model =0;
				n_regime =4;
				//value_fitness = calcule_fitness_2(which_serie_1,which_serie_1, pos_thres_1,pos_thres_2,n_regime, type_model,1);
			}
			if(codify[10]==1) {
				type_model =codify[11];
				//fprintf(cfPtr, "type_model=%d\n", type_model);
				if(type_model==0) n_regime =4;
				else n_regime =3;
				//value_fitness = calcule_fitness_2(which_serie_1,which_serie_1, pos_thres_1,pos_thres_2,n_regime, type_model,1);
			}
		}
		
	}

    //fprintf(cfPtr," AIC = %f\n",value_fitness); 
	//if(value_fitness==0.0) critter->init_fitness =0.0;
	//else critter->init_fitness = exp(-(value_fitness));
	//fprintf(cfPtr," Fitness = %f\n",exp(-(value_fitness))); 
	//fclose(cfPtr);

	free_ivector(pos_threshold,0,4);


}





