/*----------------------------------------------------------------------------*/
/* external.h - external global declarations from sga.h.                      */
/*----------------------------------------------------------------------------*/

/* If your C compiler accepts the newer-style function definitions with  */
/* prototypes included, such as:  int dummyfunc(int arg1, float arg2)    */
/* then you should leave the next line defined as is.  If not, simply    */
/* comment out the next line, or delete it.                              */
/* Some compilers do not accept certain types.  These are segregated     */
/* into a subset called SECONDARY_PROTOTYPES_ACCEPTED.  They are         */
/* accessible only if PROTOTYPES_ACCEPTED is on (ie. they are nested.)   */
/* If your compiler does accept them, and you want to have prototypes    */
/* required (s.t. the compiler barfs if you are missing a prototype for  */
/* any function) then make sure both are #defined.                       */
/* Also, this SECONDARY_PROTOTYPES_ACCEPTED define is used in checkhdr.c */
/* to add arguments to the definition of an argument-function to         */
/* ReadCheckPointHeader, for full prototype capacity.  Similarly for     */
/* WriteCheckPointHeader in the same file.  (That is, that file defines  */
/* some functions that take pointers to functions as arguments.          */
/* Currently, the prototypes for that argument only defines the return   */
/* value for the pointer function.  Full prototypes require that that    */
/* pointer-to-a-function argument have their own arguments defined.)     */
/* WARNING!  These are also defined in sga.h!                            */

#define PROTOTYPES_ACCEPTED  
#define SECONDARY_PROTOTYPES_ACCEPTED 

#include <stdlib.h>
#include <stdio.h>

#ifndef _SGA_DEFINE


#ifndef _EXTERNAL_DEFINE

#define _EXTERNAL_DEFINE
#define LINELENGTH 70   /* width of printout */
#define BITS_PER_BYTE 8 /* number of bits per byte on this machine */
#define UINTSIZE (BITS_PER_BYTE*sizeof(unsigned))   /* # of bits in unsigned */
#define MAXSCALINGWINDOW 20

/* The next 3 defines may be enlarged freely if need more fields (in a        */
/* reordering-type or mixed-type problem), or more variables in a mixed-type  */
/* representation... limited only by the memory capacity of your machine.     */
#define MAXNUMFIELDS 100

//typedef unsigned int BOOL;



// Parametri problema identificazione outliers
extern int number_oss, number_comp, order_VAR, n_oss_tot, number_rep, oss_min;
extern double **serie;
extern double **sim_Var;
extern double aic_finale;


/* file pointers */
extern FILE *outfp, *infp;

extern int type_selection, type_mutation,type_crossover, type_inversion;


/* Global structures and variables */
struct individual {
    unsigned *chrom;    /* chromosome string for the individual */
    double  fitness;    /* scaled fitness of the individual */
    double  init_fitness;       /* raw (unscaled) fitness of individual */
    long    neval;      /* number of eval. of obj. fn., incl. this indiv. */
    int     xsite[2];   /* crossover site(s) at mating (if any) */
    int     parent[2];  /* who the parents of offspring were */
    int    *utility;    /* utility field can be used as pointer to a */
		/* dynamically allocated, application-specific data structure */
};

struct bestever {
    unsigned *chrom;  /* chromosome string for the best-ever individual */
    double  fitness;  /* scaled fitness of best-ever individual of this pop */
    double  init_fitness; /* raw (unscaled) fitness of best-ever of this pop */
    long    neval;      /* number of evaluation of the Objective Function */
    int     generation; /* generation which produced it */
    int    *map_to_std; /* inversion map_to_std needed to transform this */
			/* individual's chromosome to 'standard' rep used */
			/* in the objective function.  (identity map when */
			/* not using_inversion. */
    int    *field_ends; /* also need ends of fields, for decoding best */
			/* individuals when their inversion pattern no */
			/* longer matches the current one. */
    int    *utility;    /* utility of best guy */
};

extern struct individual *oldpop;       /* last generation of individuals */

extern struct individual *newpop;       /* next generation of individuals */

extern struct individual kidholder[2];  /* temp holder for offspring      */

extern struct bestever bestfit; /* fittest individual so far, for this pop    */
				/* (if Onepop) or all subpops for THIS  */
				/* processor (if Manypops) */

extern struct bestever local_bestfit; /* for parallel subpops, best among all */
				      /* populations which THIS process (or */
				      /* processor) calculates */

extern struct bestever all_pops_bestfit; /* Fittest indiv so far, ALL pops    */
					 /* and ALL processors using this file*/
					 /* directory. */


extern unsigned *mapped_saved_chromosome; /* pointer to chromosome in current */
				/* population-specific form, for use in       */
				/* restoring chrom after objective function is*/
				/* evaluated using "unmapped, native" form    */

extern int    *this_pop_map_to_std;  /* ptr to map showing, for each field on */
		       /* this subpop's chromosomes, which field that is on   */
		       /* the "unmapped" form delivered to the objective      */
		       /* function objfunc.                                   */

extern int    *donor_pop_map_to_std; /* ptr to map showing, for each field on */
		       /* donor subpop's chromosomes, which field that is on  */
		       /* the "unmapped" form delivered to the objective      */
		       /* function objfunc.                                   */

extern int    *donor_pop_field_ends;  /* ptr to array indicating bit posn of  */
		       /* end of each field on chromosome (may be different   */
		       /* lengths, and fields may be reshuffled by inversion, */
		       /* different reps, etc. */

extern double sumfitness;       /* summed fitness for entire population */

extern double one_pop_cum_sumfitness;  /* summed fitness for whole population */
				       /* to date, only this subpopulation */

extern double one_pop_sum_best_fitness; /* sum of best fitness of each */
				 /* generation, THIS subpopulation only. */

extern long    one_pop_best_fitness_count; /* count number of times */
				    /* sumfitness has been added to */
				    /* one_pop_cum_sumfitness, which is */
				    /* number of best fitnesses calc'd. */

extern long    one_pop_online_denominator; /* track total indivs in THIS pop */
					   /* for all gens, even if popsize */
					   /* is changed. */

extern double  all_pops_sumfitness;  /* For Manypops, summed fitness for all */
				     /* individuals evaluated to date */

extern double  all_pops_sum_best_fitness;   /* For Manypops, sum of best */
				/* fitness of ALL generations across all */
				/* populations */

extern long    all_pops_online_denominator; /* For Manypops, denominator for */
						/* use in calculating on-line */
						/* performance across all */
						/* subpopulations */

extern long all_pops_best_fitness_count; /* For Manypops, count of number of */
		 /* times sumfitness is added in across ALL subpopulations   */
		 /* to date, which is same as number of times best fitness is */
		 /* calculated. */

extern long all_pops_neval; /* For Manypops, TOTAL number of evaluations */
			    /* performed to date, over ALL subpopulations */

extern double  local_cycle_sumfitness; /* For Manypops, tracks stats within */
				     /* a cycle for a single process or */
				     /* single processor.  Needed to update */
				     /* all_pops stats at end of a cycle, */
				     /* keeping good stats even across */
				     /* MULTIPLE processors or multiple */
				     /* processes on the same processor. */

extern double  local_cycle_sum_best_fitness; /* as above */

extern long    local_cycle_online_denominator; /* as above */

extern long    local_cycle_best_fitness_count; /* as above */

extern long    local_cycle_neval; /* as above */

extern int     new_bestfit_neval_within_local_cycle;

extern double  bestfit_init_fitness_within_local_cycle;

extern int     popno;  /* Number [0, npops-1] of current subpopulation; */
		       /* 0 for Onepop */

extern int     npops;  /* Number of subpopulations to simulate; 1 for Onepop */

extern int     cycle; /* Number of current cycle of subpopulations handled by */
		      /* this process or processor;  0 for Onepop */

extern int     ncycles; /* Number of cycles to run before halting (Manypops); */
			/* 0 for Onepop */

extern int     startpopnum;       /* see immediately below */

extern int     finishpopnum;      /* for TRULY parallel (multiple CPU's */
      /* runs of Manypops, tells index (in master file) of the starting and */
      /* stopping subpopulations to be calculated by THIS processor.    */

extern BOOL   single_process; /* FALSE means that more than one process */
				/* is running subpopulations on this same */ 
				/* problem in the same directory. */

extern BOOL   bestever_utilities_mallocd; /* Flag is TRUE after malloc the */
		    /* utility fields in the bestever structures declared. */
			     
extern BOOL   bestever_chroms_mallocd; /* Flag is TRUE after malloc the */
		   /* chrom fields in the bestever structures declared. */

/* Next is flag to determine whether new parameters should be read in for */
/* each subpopulation (Manypops only), or the first set read should be used */
/* for all of them (from startpopnum to finishpopnum). */
extern BOOL   all_subpops_use_same_parameters; 
			     
extern double minraw;   /* minimum raw fitness in population now */

extern double avgraw;   /* average raw fitness in population now */

extern double fit_max;  /* maximum fitness of population (after any scaling) */

extern double fit_avg;  /* average fitness of population (after any scaling) */

extern double fit_min;  /* minimum fitness of population (after any scaling) */

extern int permproblem; /* 1 indicates permutation problem; 0, otherwise. */

extern int alpha_size;  /* Number of different valid bit patterns/field */
			/* Setting it < 2 means they differ, get sizes from */
			/* array field_sizes, set in app_set_options. */

extern int numfields;   /* number of fields on the chromosome ( = length in */
			/* bits, if alpha_size = 2)                         */

extern int max_numfields_among_all_subpops; /* User sets to largest */
					    /* value for numfields used in */
					     /* ANY subpop (== numfields, */
					     /* unless different_reps == 1) */

extern int *field_sizes;    /* array defining number of alphabetic symbols in */
			 /* each field -- used to override alpha_size ONLY */
			 /* if alpha_size < 2; otherwise, ignored. */


extern int *field_ends; /* number (in [1,lchrom]) of final bit of ith field. */
			/* Note:  fields are numbered in [0, numfields-1]. */

extern int *std_field_ends; /* Ends of fields for STANDARD (initial) pattern,*/
			/* also used in fitness function.  Never changes. */

extern int incest_reduction; /* >1 if want to use Goodman's method  */
		/* for favoring crossovers among fit individuals which */
		/* are relatively distant in the Hamming sense.        */

extern BOOL user_supplied_initialization;  /* Must set TRUE in app_init() */
					   /* if user will supply own code */
					   /* for initializing individuals */


extern BOOL call_app_user_does_migration; /* Flag, if set, allows user */
					  /* full control of migration */

extern BOOL different_reps; /* Flag, if set, says different subpops  */
			    /* use different representations.  Means */
			    /* that either user is using inversion,  */
			    /* or USER MUST supply code to transform */
			    /* among different representations, in   */
			    /* routine app_transform_migrant() in    */
			    /* the user's application file.  (See    */
			    /* app_init to set flag, but it's also   */
			    /* set automatically whenever using      */
			    /* inversion.)                           */

extern BOOL using_inversion;  /* Flag, if set, says that each subpop     */
			    /* in Manypops has its own inversion         */
			    /* pattern (mapping of the chromosome),      */
			    /* and will periodically be remapped to      */
			    /* another pattern.  Implies that before     */
			    /* objective function is called, the         */
			    /* chromosome must first be "uninverted"     */
			    /* and when migration occurs, GALOPPS must   */
			    /* unmap chromosome, then remap to new       */
			    /* subpop's inversion pattern.  This flag is */
			    /* set automatically whenever pinversion is  */
			    /* non-zero, but it is also settable in      */
			    /* app_init, and user may keep it set there, */
			    /* even if pinversion is set to 0, so that   */
			    /* the program can continue using different  */
			    /* inversion patterns, even while NOT making */
			    /* any new ones.                             */
extern int parameter_changes; /* changes to values from the command line */
extern int ckptfreq;

extern int fieldlength; /* length (bits) of each field (1 for binary)  */

extern int crosstype;   /* holds code for type of crossover operator desired */

extern int mutatetype;  /* holds code for type of mutation operator desired */

extern float pcross;    /* probability of crossover */

extern float pmutation; /* probability of mutation */

extern float p_multimut; /* probability of performing a specialized, */
			  /* multiple-consecutive-locus mutation. */

extern float avg_mut_width; /* Expected value of number of loci altered by */
			    /* the multi-bit mutation operator when it does */
			    /* a mutation. */

extern float pinversion;/* probability of inversion (yielding a new map */
			/* for entire subpopulation).  Expressed as     */
			/* probability PER GENERATION.                  */

extern int autocontrol_selection; /* Boltzmann selection only; 1 means */
			/* controlling automatically each generation */
			/* using beta_s_tilde; otherwise, use user-supplied */
			/* beta, unchanged. */
extern double beta;    /* for Boltzmann selection only, default value */

extern float beta_s_tilde;   /* User-suppliable value, defaults to */
			/* 0.3 *sqrt(2.0*log(popsize)) */
			/* unless user changes.  Beta calculated from it by */
			/* dividing it by std dev. of fitness distrib. each */
			/* generation, when autocontrol_selection is true and */
			/* Boltzmann selection is active. */

extern float scalemult; /* mult of avg fitness to which most fit indiv */
			/* is scaled, so long as produces no neg values */
			/* (0 turns it off)                */

extern float conv_measure;      /* fraction of hi-fitness guys NOT closest */
				/* (Hamming) to best current individual    */

extern float   percent_lost;   /* percent of loci with all 0's or all 1's */

extern float   percent_converged; /* pct of loci with >=95% 0's or >= 95% 1's */

extern float   avg_bias; /* average bias (percent same allele) of all loci */

extern double  on_line_performance; /* Average of ALL fitnesses of ALL */
			     /* individuals evaluated over entire run (for */
			     /* one subpopulation, if IPGA) */

extern double  off_line_performance; /* Average of BEST individual's fitness */
			      /* for ALL generations in entire run (for one */
			      /* subpopulation, if Manypops) */

extern double  all_pops_on_line_performance; /* Average on_line_performance */
					     /* across ALL subpops (should  */
					     /* not be used by Onepop, only */
					     /* by Manypops)                */

extern double  all_pops_off_line_performance;/* Average off_line_performance */
					     /* across ALL subpops (should   */
					     /* not be used by Onepop, only  */
					     /* by Manypops)                 */
extern float conv_sigma_coeff;  /* mult of sigma; guys with fitness above  */
				/* this no. of std dev'ns above mean are   */
				/* examined for sim to best guy, using     */
				/* Hamming distance.                       */

extern float sigma_trunc;       /* multiplier for sigma, per Goldberg,p.125 */

extern float sigma;     /* std deviation for current population */

extern long neval;      /* number of eval'ns of the Objective Function, this
			 * pop */

extern int numfiles;    /* number of files specified on cmd line */
extern int interactive;
extern int extending;   /* starting a continuation run from a previous state */
extern int popsize;     /* population size */

extern int lchrom;      /* length of the chromosome per individual */

extern int chromsize;   /* number of uns ints needed to store lchrom string */

extern int max_chromsize_among_all_subpops; /* User sets to largest chromsize */
					 /* used in ANY subpop (== chromsize, */
					 /* unless different_reps == 1) */

extern int genspercycle;        /* num. of generations to run island parallel
				 * GA before checkpointing        */

extern int startgen;    /* number of starting gen (!=0 for restart)    */

extern int gen;         /* current generation number */

extern int maxgen;      /* maximum generation number */

extern int run;         /* current run number */

extern int maxruns;     /* maximum number of runs to make */

extern int tourneysize; /* size of tournament if tselect is in use (def.=2). */
			/* Otherwise, ignored. */

extern int printstrings;        /* flag to print chromosome strings (default
				 * on) */

extern long nmutation;  /* number of mutations  */

extern long ncross;     /* number of crossovers  */

extern int bestnow;     /* index of best individual of current generation */

extern int *ones;       /* Array counting 1's at each locus, to look    */
			/* for misaligns, convergence, etc. */

extern int convinterval;        /* generations between reporting convergence
				 * statistics (number of ones per locus) */

extern int crowding_factor;     /* for DeJong-style crowding, 0 means none    */

extern int stochastic;  /* if 1, all chroms. eval'd every generation */

extern int elitism;     /* if 1, best of gen. always present in next gen. */

extern int quiet;       /* suppresses most output except from app. routines */

extern int scaling_window;      /* size of scaling window(-1 = no window
				 * scaling) */

extern int windowstart; /* pointer for window scaling */

extern int windowend;   /* pointer for window scaling */

extern double savedmins[MAXSCALINGWINDOW];      /* min fitness history, for   */                                                /* window scaling */

extern int *available;  /* dynamically allocated in memory.c */

extern int *evaluate;   /* malloc popsize flags saying need to call objfunc. */

/* Here are variables to record the status of what operators, selection */
/* routines, etc. are compiled in, for reporting to the user. Values are */
/* determined by files compiled in at run time. */
extern char   *which_selection;
extern char   *which_crossover;
extern char   *which_mutation;
extern char   *which_inversion;

/* Below are some variables used by various permutation-type operators, for   */
/* which space is malloc'ed only if permutation-type problem being run.       */
/* CAUTION:  If you load permutation-type operators, you MUST answer 'y' to   */
/* the question "Permutation-type problem?"                                   */
extern int *intarray1;

extern int *intarray2;

extern int *intarraybak1;

extern int *intarraybak2;

extern int *randomlst;

extern int *tmparray;


/* File name arrays */
extern char restartfileprefix[29], restarthdrfilename[13]; 

extern char restartindfilename[13]; 

extern char masterfileprefix[29], masterfilename[13];

extern char checkpthdrfilename[13], checkptindfilename[13]
	    , checkptfileprefix[29];


/* variables for random number generator, so can preserve them for restart */
extern double oldrand[55];
extern int jrand;
extern double rndx2;
extern int rndcalcflag;
extern float randomseed;

/* For optional superuniform initialization */
extern int superuniform_cyclebitwidth;
extern int superuniform_initialization;

extern int many;

/* Application-dependent external declarations go after here...  */

#endif

#endif

#ifdef PROTOTYPES_ACCEPTED
#include "sgafunc.h"

#else
#include "sgapure.h"

#endif


