/*----------------------------------------------------------------------------*/
/* memory.c - memory management routines for GALOPPS code                     */
/*----------------------------------------------------------------------------*/

#include "stdafx.h"
#include "MSetar.h"
#include "MSetarDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#include <stdlib.h>
#include "external.h"

unsigned *chrombuffer;


/* memory allocation of space for global data structures */
void initmalloc() {

	unsigned nbytes, *chrompointer;
	int     j;

    /* memory for old and new populations of individuals */
    nbytes = popsize * sizeof(struct individual);
    if ((oldpop = (struct individual *) malloc(nbytes)) == NULL) nomemory("oldpop");
	if ((newpop = (struct individual *) malloc(nbytes)) == NULL) nomemory("newpop");

	/* memory for chromosome strings in populations */
    nbytes = 2* popsize * chromsize * sizeof(unsigned);
    if ((chrombuffer = (unsigned *) malloc(nbytes)) == NULL) nomemory("Space for ALL chromosomes");

	chrompointer = chrombuffer;

	for (j = 0; j < popsize; j++) {
        oldpop[j].chrom = chrompointer;
        chrompointer += chromsize;
        newpop[j].chrom = chrompointer;
        chrompointer += chromsize;
	}

	/* allocate memory for an additional chromosome for temp usage */
//    if ((mapped_saved_chromosome = (unsigned *) malloc(nbytes)) == NULL) nomemory("mapped_saved_chromosome");

    /* allocate memory needed for inversion operations, even if not in use.  */
//    nbytes = numfields * sizeof(int);
//    if ((donor_pop_map_to_std = (int *) malloc(nbytes)) == NULL) nomemory("donor_pop_map_to_std");

	/* allocate memory needed for inversion operations, even if not in use.  */
//    nbytes = numfields * sizeof(int);
//    if ((donor_pop_field_ends = (int *) malloc(nbytes)) == NULL) nomemory("donor_pop_field_ends");

	/* allocate memory for popsize counters of ones, for tracking       */
    /* convergence of population, correctness of initialization, etc.    */
    nbytes = lchrom * sizeof(int);
    if ((ones = (int *) malloc(nbytes)) == NULL) nomemory("ones");

	nbytes = popsize * sizeof(int);
    if ((evaluate = (int *) malloc(nbytes)) == NULL)  nomemory("evaluate");
    if ((available = (int *) malloc(nbytes)) == NULL) nomemory("available");
  
	nbytes = chromsize * sizeof(unsigned);
    if ((kidholder[0].chrom = (unsigned *) malloc(nbytes)) == NULL) nomemory("kidholder");
    if ((kidholder[1].chrom = (unsigned *) malloc(nbytes)) == NULL) nomemory("kidholder");

	/* If not already done, allocate memory for array of field sizes */
    if (field_sizes == NULL) {
        nbytes = numfields * sizeof(int);
        if ((field_sizes = (int *) malloc(nbytes)) == NULL) nomemory("evaluate");
	}

	/* and allocate memory for array of field ending bit numbers */
    if (field_ends == NULL) {
        nbytes = numfields * sizeof(int);
        if ((field_ends = (int *) malloc(nbytes)) == NULL) nomemory("evaluate");
	}

	/* allocate any auxiliary memory for selection */
    if(type_selection==0) select_memory_rank();
	if(type_selection==1) select_memory_roulette_wheel();
	if(type_selection==2) select_memory_SUS();
	if(type_selection==3) select_memory_tournament();
	if(type_selection==4) select_memory_stochastic_remainder();

	
}


/* fields may be of different lengths.  Then need early malloc of arrays */
/* below.  (Also called during checkpoint header reading in checkhdr.c  */
/* on continuation run. */

void malloc_field_sizes_arrays() {

	unsigned nbytes;

	/* If not already done, allocate memory for array of field sizes */
	if (field_sizes == NULL) {
		nbytes = numfields * sizeof(int);
		if ((field_sizes = (int *) malloc(nbytes)) == NULL)
			nomemory("field_sizes");
	}

	/* allocate memory for array of field ending bit numbers */
	if (field_ends == NULL) {
		nbytes = numfields * sizeof(int);
		if ((field_ends = (int *) malloc(nbytes)) == NULL)
			nomemory("field_ends");
	}

	/* allocate memory for array of field ending bit numbers in */ 
    /* the STANDARD (initialized) inversion pattern used in fitness */
    /* function.  This is assumed to be the pattern used in Subpopulation 0, */
    /* at the first generation of the run.  (May be too long, but never too */
    /* short. )  Important whenever different_reps is set, whether for */
    /* inversion or some non-inversion representation difference among subpops. */
    /* NOTE:  This is the only one of these mallocs NEVER FREED -- it must be */
    /* preserved across all subpopulations.  However, since it checks to see */
    /* if it has already been malloc'd, it will only be malloc'd once. */ 
	
/*	
	if (std_field_ends == NULL) {
		nbytes = max_numfields_among_all_subpops * sizeof(int);
		if ((std_field_ends = (int *) malloc(nbytes)) == NULL)
			nomemory("std_field_ends");
	}
*/
	

}


/* Malloc the variable (chrom and map_to_std and utility) fields for the */
/* bestever structures.  Usually, sizes are chromsize and numfields and */
/* GetUtilitySize(), but if different_reps flag is set, user must (in */
/* app_set_options) set max_chromsize_among_all_subpops to its largest value, */
/* and similarly set max_numfields_among_all_subpops to its largest value, */
/* so that any subpop's chrom, map_to_std, and utility fields may be */
/* stored in checkpoint files, etc. */
/* These variables are never freed until the end of the run. */

void bestever_variable_malloc() {

    int nbytes, i;
    static int firsttimecalled = 1;

    if (firsttimecalled) {
		firsttimecalled = 0;
       /* Allocate variable memory structures for "bestfit"-type tracking, */
       /* (their chrom fields), substituting larger block when needed. */

		nbytes = max_chromsize_among_all_subpops * sizeof(unsigned);

        if ((bestfit.chrom = (unsigned *) malloc(nbytes)) == NULL) nomemory("bestfit chromosome");

//        if ((local_bestfit.chrom = (unsigned *) malloc(nbytes)) == NULL) nomemory("local_bestfit chromosome");

//        if ((all_pops_bestfit.chrom = (unsigned *) malloc(nbytes)) == NULL) nomemory("all_pops_bestfit chromosome");

		/* Allocate variable memory structures for "bestfit"-type tracking, */
        /* (their map_to_std fields), substituting larger block when needed. */



        nbytes = max_numfields_among_all_subpops * sizeof(int);
//	    if ((bestfit.map_to_std = (int *) malloc(nbytes)) == NULL) nomemory("bestfit map_to_std");

//      if ((local_bestfit.map_to_std = (int *) malloc(nbytes)) == NULL) nomemory("local_bestfit map_to_std");

//      if ((all_pops_bestfit.map_to_std = (int *) malloc(nbytes)) == NULL) nomemory("all_pops_bestfit map_to_std");
    
//      if ((bestfit.field_ends = (int *) malloc(nbytes)) == NULL) nomemory("bestfit field_ends");

//        if ((local_bestfit.field_ends = (int *) malloc(nbytes)) == NULL) nomemory("local_bestfit field_ends");

//        if ((all_pops_bestfit.field_ends = (int *) malloc(nbytes)) == NULL) nomemory("all_pops_bestfit field_ends");

		/* Call routine to allocate any "utility" memory */
        /* needed for each bestever, pointed to by its utility field. */
        //sys_util_malloc_bestever_utility();
	}
	
}



/* A routine to free all the space dynamically allocated in initmalloc() */
/* and malloc_field_sizes_arrays(). */
void freeall() {

	int     i;

    /* call to sys_util_free() routine frees memory of utility pointer */
    /* Note: must be done before the free's below, or */
    /* the pointer to utility fields is lost.         */
    sys_util_free();

    free (chrombuffer);
    chrombuffer = NULL;

    free(oldpop);
    oldpop = NULL;
    free(newpop);
    newpop = NULL;
    free(evaluate);
    evaluate = NULL;
    free(ones);
    ones = NULL;
    free(available);
    available = NULL;
    free(kidholder[0].chrom);
    kidholder[0].chrom = NULL;
    free(kidholder[1].chrom);
    kidholder[1].chrom = NULL;
    free (mapped_saved_chromosome);
//    mapped_saved_chromosome = NULL;
//    free (this_pop_map_to_std);
    this_pop_map_to_std = NULL;
//    free (donor_pop_map_to_std);
    donor_pop_map_to_std = NULL;
    free (field_sizes);
    field_sizes = NULL;
    free (field_ends);
    field_ends = NULL;

    /* free any auxiliary memory needed for selection */
    if(type_selection==0) select_free_rank();
	if(type_selection==1) select_free_roulette_wheel();
	if(type_selection==2) select_free_SUS();
	if(type_selection==3) select_free_tournament();
	if(type_selection==4) select_free_stochastic_remainder();

}


void nomemory(char *string) {

	fprintf(outfp, "malloc: out of memory making %s!!\n", string);
    exit(-1);

}


void MallocPermOperSpace() {

	if (intarray1 == NULL) {
        if ((intarray1 = (int *) malloc(numfields * sizeof(int))) == NULL)
            nomemory("MallocPermOperSpace.");
	}

	if (intarray2 == NULL) {
		if ((intarray2 = (int *) malloc(numfields * sizeof(int))) == NULL)
            nomemory("MallocPermOperSpace.");
	}

	if (intarraybak1 == NULL) {
        if ((intarraybak1 = (int *) malloc(numfields * sizeof(int))) == NULL)
            nomemory("MallocPermOperSpace.");
	}
  
	if (intarraybak2 == NULL) {
        if ((intarraybak2 = (int *) malloc(numfields * sizeof(int))) == NULL)
            nomemory("MallocPermOperSpace.");
	}

	if (randomlst == NULL) {
      if ((randomlst = (int *) malloc(numfields * sizeof(int))) == NULL)
          nomemory("MallocPermOperSpace.");
	}
    
	if (tmparray == NULL) {
        if ((tmparray = (int *) malloc(numfields * sizeof(int))) == NULL)
            nomemory("MallocPermOperSpace.");
	}
}


/* application dependent utility field malloc() calls called at initialization */
/* during user input and upon restart from a saved checkpoint file.  */
void sys_util_malloc() {

	unsigned nbytes;
    int     i;

    nbytes = GetUtilitySize();
    if(nbytes > 0) {
		for (i = 0; i < popsize; i++) {
			if ((newpop[i].utility = (int *) malloc(nbytes)) == NULL)
				nomemory("newpop utility");
			if ((oldpop[i].utility = (int *) malloc(nbytes)) == NULL)
				nomemory("oldpop utility");
		}
	}
}


/* This routine frees any utility field memory allocated in sys_util_malloc, */
/* called by freeall() */
void sys_util_free() {

    int     i;

    if (GetUtilitySize() > 0) {
		for (i = 0; i < popsize; i++) {
			free(newpop[i].utility);
	        newpop[i].utility = NULL;
	        free(oldpop[i].utility);
	        oldpop[i].utility = NULL;
		}
    }
}









